package sat;
import javax.xml.parsers.*;
import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;
import java.util.*;

// Copyright (c) 2002 Rick Mugridge, University of Auckland.
// Released under the terms of the GNU General Public License version 2 or later.

   /** This processes the XML using a DOM.
       @author rick mugridge, september 2002
   */
public class ProcessXML {
        /** This needs to deal with "\n" carefully, because the XML could be laid out
	    with white space separating nodes.  So some of the white space needs to be
	    ignored.
	*/
    public static String getNodeAsString(Element element) {
	String result = "<"+element.getNodeName();
	result += getAttributesAsString(element.getAttributes());
	NodeList children = element.getChildNodes();
	if (children.getLength() == 0)
	    return result+"/>";
	result += ">";
	for (int i = 0; i < children.getLength(); i++) {
	    Node node = children.item(i);
	    if (node instanceof Element)
		result += getNodeAsString((Element)node);
	    else if (node instanceof Text)
		result += getValueWithSpecialsEscaped(node.getNodeValue());
	}
	return result+"</"+element.getNodeName()+">";
    }
    protected static String getAttributesAsString(NamedNodeMap map) {
	String result = "";
	int attributeCount = 0;
	for (int i = 0; i < map.getLength(); i++) {
	    Attr attr = (Attr)map.item(i);
	    if (attr.getSpecified())
		result += " "+attr.getName()+"=\""+getValueWithSpecialsEscaped(attr.getValue())+"\"";
	}
	return result;
    }
    public static String getValueWithSpecialsEscaped(String s) {
	if (whiteSpace(s))
	    return "";
	String result = s;
	final String[] specialChars = {"&","<",">","\"","'"};
	final String[] escapeString = {"&amp;", "&lt;", "&gt;", "&quot;", "&apos;"};
	for (int i = 0; i < specialChars.length; i++) {
	    String special = specialChars[i];
	    int pos = -1;
	    while (true) {
		pos = result.indexOf(special,pos+1);
		if (pos < 0)
		    break;
		result = result.substring(0,pos)+escapeString[i]+result.substring(pos+special.length());
	    }
	}
	return result;
    }
    protected static boolean whiteSpace(String s) {
	for (int i = 0; i < s.length(); i++) {
	    char ch =  s.charAt(i);
	    if (ch != ' ' && ch != '\n'&& ch != '\r' && ch != '\t')
		return false;
	}
	return true;
    }


    public static boolean same(String expectedString, String actualString) {
	try {
	    Element expectedNode = getDoc(expectedString);
	    Node actualNode = getDoc(actualString);
	    return same(expectedNode,actualNode);
	}
	catch (ParserConfigurationException ex) {
	}
	catch (SAXException ex) {
	}
	catch (IOException ex) {
	}
	return false;
    }
    public static boolean same(Node expectedNode, Node actualNode) {
	if (expectedNode instanceof Element)
	    return (actualNode instanceof Element) && 
		expectedNode.getNodeName().equals(actualNode.getNodeName()) &&
		attributesSame(expectedNode,actualNode) && 
		childrenSame(expectedNode,actualNode);
	if (expectedNode instanceof Text)
	    return (actualNode instanceof Text) &&
		expectedNode.getNodeValue().equals(actualNode.getNodeValue());
	if (expectedNode instanceof CDATASection)
	    return (actualNode instanceof CDATASection) &&
		expectedNode.getNodeValue().equals(actualNode.getNodeValue());
	return expectedNode.getNodeName().equals(actualNode.getNodeName());
    }
    protected static boolean attributesSame(Node n1, Node n2) {
	NamedNodeMap a1 = n1.getAttributes();
	NamedNodeMap a2 = n2.getAttributes();
	if (a1 == null)
	    return (a1 == null);
	else if (a2 == null)
	    return false;
	if (a1.getLength() != a2.getLength())
	    return false;
	for (int i = 0; i < a1.getLength(); i++) {
	    Attr attr1 = (Attr)a1.item(i);
	    Attr attr2 = (Attr)a2.getNamedItem(attr1.getName());
	    if (attr2 == null)
		return false;
	    if (!attr1.getValue().equals(attr2.getValue()))
		return false;
	}
	return true;
    }
    protected static boolean childrenSame(Node n1, Node n2) {
	NodeList nodes1 = n1.getChildNodes();
	NodeList nodes2 = n2.getChildNodes();
	if (nodes1.getLength() != nodes2.getLength())
	    return false;
	for (int i = 0; i < nodes1.getLength(); i++)
	    if (!same(nodes1.item(i),nodes2.item(i)))
		return false;
	return true;
    }
    public static Element getDoc(String s) throws ParserConfigurationException, 
						  SAXException, IOException {
	DocumentBuilder builder = 
	    DocumentBuilderFactory.newInstance().newDocumentBuilder();
	Document doc = builder.parse(new StringBufferInputStream(
		"<?xml version=\"1.0\" encoding=\"ISO-8859-1\" standalone=\"yes\" ?>"+s));
	return doc.getDocumentElement();
	}
}
