package sat;
import sat.*;
import fit.*;
import java.io.*;
import java.util.*;

// Copyright (c) 2002 Rick Mugridge, University of Auckland.
// Released under the terms of the GNU General Public License version 2 or later.

public class Sat {
    protected Server server;
    protected int tracingLevel = 0;
    protected String[] clientNames;
    protected List clients = new ArrayList();
    protected final String host = "localhost";
    protected int timeOut;
    protected int postSendDelay;

    public Sat(String serverClassName, int timeOut, 
	       int postSendDelay, int tracingLevel) throws Exception {
	this.clientNames = clientNames;
	this.timeOut = timeOut;
	this.postSendDelay = postSendDelay;
	this.tracingLevel = tracingLevel;
	server = (Server)Class.forName(serverClassName).newInstance();
	while (!server.isRunning()) {
	    Thread.sleep(50);
	}
	if (tracingLevel > 0)
	    System.out.println("Start server: "+serverClassName+
			       " on port "+server.getPortNo());
    }
      /** Affects the time-out value for all clients that are created subsequently
       */
    public void setTimeOut(int timeOut) {
	this.timeOut = timeOut;
    }
      /** Affects the delay that occurs after subsequent sends
       */
    public void setPostSendDelay(int postSendDelay) {
	this.postSendDelay = postSendDelay;
    }
    public void setTracingLevel(int tracingLevel) {
	this.tracingLevel = tracingLevel;
    }
    public boolean isRunning() {
	return server.isRunning();
    }
    public void send(int clientNo, String msg) throws IOException {
	if (tracingLevel > 0)
	    System.out.println("Send to #"+clientNo+": "+msg);
	getClient(clientNo).send(msg);
	if (postSendDelay > 0) {
	    try {
		Thread.sleep(postSendDelay);
	    } catch (InterruptedException ex) {
	    }
	}
    }
    public String receive(int clientNo) throws IOException {
	String msg = getClient(clientNo).receive();
	if (tracingLevel > 0)
	    System.out.println("Rx from #"+clientNo+": "+msg);
	return msg;
    }
    protected ClientSocket getClient(int clientNo) throws IOException {
	while (clientNo >= clients.size())
	    clients.add(new ClientSocket(host,server.getPortNo(),
					 "client#"+clientNo,timeOut));
	return (ClientSocket)clients.get(clientNo);
    }
    public void closeClients() throws IOException {
	for (Iterator it = clients.iterator(); it.hasNext();) {
	    ClientSocket client = (ClientSocket)it.next();
	    client.close();
	}
    }
    public void close(int clientNo) throws IOException {
	getClient(clientNo).close();
    }
}
