package sat.fit;
import sat.*;
import fit.*;

// Copyright (c) 2002 Rick Mugridge, University of Auckland.
// Released under the terms of the GNU General Public License version 2 or later.

    /** A fixture for running sats (Socket Acceptance Tests)
	@author rick mugridge, october 2002
    */
public class SatFixture extends ActionRowFixture {
    protected Sat sat = null;
    protected int timeOut = 500;
    protected int tracingLevel = 0;
    protected int postSendDelay = 50;

    public void timeOut(Parse cells) {
	try {
	    timeOut = new Integer(cells.more.text()).intValue();
	    if (sat != null)
		sat.setTimeOut(timeOut);
	    right(cells);
	}
	catch (Exception ex) {
	    exception(cells.more,ex);
	}
    }
    public void startServer(Parse cells) {
	try {
	    sat = new Sat(cells.more.text(),timeOut,
			  postSendDelay,tracingLevel);
	    right(cells);
	}
	catch (Exception ex) {
	    exception(cells.more,ex);
	}
    }
    public void stopServer(Parse cells) {
	try {
	    sat.closeClients();
	    for (int i = 0; i < 40 && sat.isRunning(); i++)
		Thread.sleep(50);
	    if (sat.isRunning())
		wrong(cells,"Server failed to finish");
	    else
		right(cells);
	} catch (Exception ex) {
	    exception(cells,ex);
	}
    }
    protected final ColumnCommand sendCommand = new ColumnCommand() {
	    public void execute(int column, Parse cell) throws Exception {
		sat.send(column,preProcessMsg(cell.text()));
		right(cell);
	    }
	};
    public void send(Parse cells) {
	processColumns(cells,sendCommand);
    }
    protected final ColumnCommand rxCommand = new ColumnCommand() {
	    public void execute(int column, Parse cell) throws Exception {
		String reply = sat.receive(column);
		if (equalMessages(reply,preProcessMsg(cell.text())))
		    right(cell);
		else
		    wrong(cell,reply);
	    }
	};
    public void rx(Parse cells) {
	processColumns(cells,rxCommand);
    }
        /** This is overridden when XML is being processed.
	 */
    protected String preProcessMsg(String msg) throws Exception {
	return msg;
    }
    protected final ColumnCommand sendRefusedCommand = new ColumnCommand() {
	    public void execute(int column, Parse cell) throws Exception {
		try {
		    sat.send(column,preProcessMsg(cell.text()));
		    wrong(cell);
		} catch (java.net.ConnectException ex) {
		    right(cell);
		}
	    }
	};
    public void sendRefused(Parse cells) {
	processColumns(cells,sendRefusedCommand);
    }
    public void delay(Parse cells) {
	try {
	    Thread.sleep(new Integer(cells.more.text()).intValue());
	    right(cells);
	}
	catch (Exception ex) {
	    exception(cells.more,ex);
	}
    }
    public void postSendDelay(Parse cells) {
	try {
	    postSendDelay = new Integer(cells.more.text()).intValue();
	    if (sat != null)
		sat.setPostSendDelay(postSendDelay);
	    right(cells);
	}
	catch (Exception ex) {
	    exception(cells.more,ex);
	}
    }
    protected boolean equalMessages(String expected, String actual) {
	return expected.equals(actual);
    }
    protected final ColumnCommand rxDiffersCommand = new ColumnCommand() {
	    public void execute(int column, Parse cell) throws Exception {
		String reply = sat.receive(column);
		if (equalMessages(reply,preProcessMsg(cell.text())))
		    wrong(cell,"Are equal");
		else
		    right(cell);
	    }
	};
    public void rxDiffers(Parse cells) {
	processColumns(cells,rxDiffersCommand);
    }
    protected final ColumnCommand rxTimesOutCommand = new ColumnCommand() {
	    public void execute(int column, Parse cell) throws Exception {
		try {
		    String reply = sat.receive(column);
		    wrong(cell,"Did not timeout");
		} catch (java.net.SocketTimeoutException ex) {
		    right(cell);
		}
	    }
	};
    public void rxTimesOut(Parse cells) {
	processColumns(cells,rxTimesOutCommand);
    }
    protected final ColumnCommand rxRefusedCommand = new ColumnCommand() {
	    public void execute(int column, Parse cell) throws Exception {
		try {
		    String reply = sat.receive(column);
		    wrong(cell,"Connected OK");
		} catch (java.net.ConnectException ex) {
		    right(cell);
		}
	    }
	};
    public void rxRefused(Parse cells) {
	processColumns(cells,rxRefusedCommand);
    }
    protected final ColumnCommand closeCommand = new ColumnCommand() {
	    public void execute(int column, Parse cell) throws Exception {
		sat.close(column);
		right(cell);
	    }
	};
    public void close(Parse cells) {
	processColumns(cells,closeCommand);
    }
    public void comment(Parse cells) {
    }
    public void traceOn(Parse cells) {
	setTracingLevel(1);
    }
    public void traceOff(Parse cells) {
	setTracingLevel(0);
    }
    protected void setTracingLevel(int tracingLevel) {
	this.tracingLevel = tracingLevel;
	if (sat != null)
	    sat.setTracingLevel(tracingLevel);
    }
}
