package sat.mockServer;
import sat.*;
import java.io.*;
import java.net.*;

// Copyright (c) 2002 Rick Mugridge, University of Auckland.
// Released under the terms of the GNU General Public License version 2 or later.

   /** Provides some general capability for running mock servers for 
       testing.
       @author rick mugridge, september 2002
   */
public abstract class AbstractServer extends Thread implements Server {
    protected static final int DEBUG = 0;
    protected int portNo = 9999;
    protected boolean running = false;

    public AbstractServer() {
	this(9999);
    }
    public AbstractServer(int portNo) {
	this.portNo = portNo;
	start();
    }
    public void run() {
	running = true;
	if (DEBUG>0)
	    System.out.println("Server started");
	try {
	    ServerSocket server = new ServerSocket(getPortNo());
	    server.setSoTimeout(5000);
	    manageConnections(server);
	    server.close();
	}
	//	catch(SocketTimeoutException ex) {
	//	    throw new RuntimeException("Server timed out");
	//	}
	catch(InterruptedIOException ex) {
	    throw new RuntimeException("Server timed out");
	}
	catch(IOException ex) {
	    throw new RuntimeException("Server problem: "+ex);
	}
	if (DEBUG>0)
	    System.out.println("Server finished");
	running = false;
    }
    public int getPortNo() {
	return portNo;
    }
    public boolean isRunning() {
	return running;
    }
    public abstract void manageConnections(ServerSocket server) throws IOException;
}
