package sat.mockServer;
import java.io.*;
import java.net.*;

// Copyright (c) 2002 Rick Mugridge, University of Auckland.
// Released under the terms of the GNU General Public License version 2 or later.

  /** An abstraction for a (testing) server thread that handles all 
      the usual setup, etc.
      A subclass simply provides the check() method.
      @author rick mugridge Sept 2002
  */
public abstract class ServerThread extends Thread {
    protected Socket socket;
    protected String name;
    protected boolean debug = false;
    protected BufferedReader in;
    protected PrintStream out;

    public ServerThread(Socket socket, String name) {
	this(socket,name,false);
    }
    public ServerThread(Socket socket, String name, boolean debug) {
	this.socket = socket;
	this.name = name;
	this.debug = debug;
	start();
    }
    public void run() {
	if (debug)
	    System.out.println(name+" running...");
	try {
	    socket.setSoTimeout(6000);
	    in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
	    out = new PrintStream(socket.getOutputStream());
	    if (debug)
		System.out.println(name+" reading...");
	    check(in,out);
	    out.close();
	    in.close();
	    socket.close();
	}
	catch(InterruptedIOException ex) {
	    //	catch(SocketTimeoutException ex) {
	    System.out.println(name+" timed out");
	}
	catch(IOException ex) {
	    throw new RuntimeException(name+" server problem: "+ex);
	}
	finally { // In case the following two methods are called out of run()
	    in = null;
	    out = null;
	}
    }
    public String readLine() throws IOException {
	return in.readLine();
    }
    /** Only call once the Thread is run()ing */
    protected boolean expectedMessage(String msg) throws IOException {
	String line = in.readLine();
	if (line == null || !line.equals(msg)) {
	    out.println("Expected '"+msg+"' but got '"+line+"'");
	    return false;
	}
	return true;
    }
    /** Only call once the Thread is run()ing */
    protected void expectShutdown() throws IOException {
	expectedMessage("<Shutdown/>");
    }
    protected abstract void check(BufferedReader in, PrintStream out) throws IOException;
}
